(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'immutable', '../lib/schema', '../store'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('immutable'), require('../lib/schema'), require('../store'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.immutable, global.schema, global.store);
    global.store = mod.exports;
  }
})(this, function (exports, _immutable, _schema, _store) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.default = initApiStore;

  var _store2 = _interopRequireDefault(_store);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  var _slicedToArray = function () {
    function sliceIterator(arr, i) {
      var _arr = [];
      var _n = true;
      var _d = false;
      var _e = undefined;

      try {
        for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
          _arr.push(_s.value);

          if (i && _arr.length === i) break;
        }
      } catch (err) {
        _d = true;
        _e = err;
      } finally {
        try {
          if (!_n && _i["return"]) _i["return"]();
        } finally {
          if (_d) throw _e;
        }
      }

      return _arr;
    }

    return function (arr, i) {
      if (Array.isArray(arr)) {
        return arr;
      } else if (Symbol.iterator in Object(arr)) {
        return sliceIterator(arr, i);
      } else {
        throw new TypeError("Invalid attempt to destructure non-iterable instance");
      }
    };
  }();

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
    return typeof obj;
  } : function (obj) {
    return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
  };

  function _toConsumableArray(arr) {
    if (Array.isArray(arr)) {
      for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) {
        arr2[i] = arr[i];
      }

      return arr2;
    } else {
      return Array.from(arr);
    }
  }

  /* basic state tree layout
  * {
  *   $api: {
  *     [url]: {
  *       [resource]: {
  *         models: {
  *           [id]: {
  *             [key]: any,
  *             _fetched?: boolean
  *           }
  *         },
  *         requests: {
  *           [url]: {
  *             pending?: boolean,
  *             error?: Object,
  *             data?: List | Map,
  *             promise?: Promise<any>,
  *             format?: string
  *           }
  *         }
  *       }
  *     }
  *   }
  * }
  */

  function initApiStore(url, schema) {
    var store = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _store2.default;


    var resourceType = schema.title;
    var rootPath = ['$api', url, resourceType];

    var methods = {

      // core

      _getState: store.getState,

      getState: function getState(key) {
        return store.getState(rootPath.concat(key || []).map(function (item) {
          return String(item);
        }));
      },

      setState: function setState(val, key) {
        return store.setState(rootPath.concat(key || []).map(function (item) {
          return String(item);
        }), val);
      },

      setStateQuiet: function setStateQuiet(val, key) {
        return store.setStateQuiet(rootPath.concat(key || []), val);
      },

      // requests

      requestsPath: function requestsPath(path) {
        return ['requests'].concat(path || []);
      },

      getRequests: function getRequests(path) {
        return methods.getState(methods.requestsPath(path));
      },

      getRequestsData: function getRequestsData(path) {
        return methods.getRequests([path, 'data']);
      },

      getRequestsFormat: function getRequestsFormat(path) {
        return methods.getRequests([path, 'format']);
      },

      setRequests: function setRequests(data, path) {
        return methods.setState(data, methods.requestsPath(path));
      },

      setRequestsData: function setRequestsData(path, data) {
        return methods.setRequests(data, [path, 'data']);
      },

      setRequestsFormat: function setRequestsFormat(path, data) {
        return methods.setRequests(data, [path, 'format']);
      },

      getRequestsPromise: function getRequestsPromise(path) {
        return methods.getRequests([path, 'promise']);
      },

      setRequestsPromise: function setRequestsPromise(path, data) {
        return methods.setStateQuiet(data, methods.requestsPath([path, 'promise']));
      },

      getPending: function getPending(path) {
        return methods.getRequests([path, 'pending']);
      },

      setPending: function setPending(path, data) {
        return methods.setStateQuiet(data, methods.requestsPath([path, 'pending']));
      },

      getError: function getError(path) {
        return methods.getRequests([path, 'error']);
      },

      setError: function setError(path, error) {
        var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

        var method = options.quiet ? methods.setStateQuiet : methods.setState;
        method(error, methods.requestsPath([path, 'error']));
      },

      // models

      getModels: function getModels() {
        return methods.getState('models') || (0, _immutable.Map)();
      },

      setModels: function setModels(data) {
        return methods.setState(data, 'models');
      },

      getModel: function getModel(id) {
        return methods.getState(['models', id]);
      },

      setModel: function setModel(id, data) {
        return methods.setState(data, ['models', id]);
      },

      deleteModel: function deleteModel(id) {
        var undo = [];
        var idStr = String(id);
        methods.setState((methods.getState() || (0, _immutable.Map)()).withMutations(function (map) {
          var model = methods.getModel(idStr);
          if (model) {
            map.update('models', function () {
              var models = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.Map)();
              return models.delete(idStr);
            });
            undo.push(function () {
              return methods.setModels(methods.getModels().set(idStr, model));
            });
            undo.push.apply(undo, _toConsumableArray(methods.removeFromCollections(map, idStr)));
          }
        }));
        return undo;
      },

      updateModel: function updateModel(id, vals) {
        var undo = [];
        var model = methods.getModel(id);
        if (model) {
          methods.setModel(id, model.mergeDeep(vals));
          undo.push(function () {
            return methods.setModels(methods.getModels().set(id, model));
          });
        }
        return undo;
      },

      // collections (lists hydrated with models)

      getCollection: function getCollection() {
        var path = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '/' + resourceType;

        var collection = methods.getRequestsData(path);
        if (collection) {
          if (methods.getRequestsFormat(path) === 'string') {
            return collection;
          } else {
            var models = methods.getModels();
            return collection.map(function (id) {
              return models.get(id) || (0, _immutable.Map)();
            });
          }
        } else {
          return null;
        }
      },

      setCollection: function setCollection() {
        var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.List)();
        var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '/' + resourceType;

        methods.setState((methods.getState() || (0, _immutable.Map)()).withMutations(function (map) {
          if (data.length && _typeof(data[0]) !== 'object') {
            map.setIn(methods.requestsPath([path, 'data']), (0, _immutable.List)(data));
            map.setIn(methods.requestsPath([path, 'format']), 'string');
          } else {
            var dict = data.reduce(function (memo, item) {
              return _extends({}, memo, _defineProperty({}, (0, _schema.getIdFromModel)(item), item));
            }, {});
            var collection = (0, _immutable.List)(Object.keys(dict));
            map.update('models', function () {
              var models = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.Map)();
              return models.mergeDeep(dict);
            });
            map.setIn(methods.requestsPath([path, 'data']), collection);
          }
        }));
        return methods.getCollection(path);
      },

      updateCollection: function updateCollection(model) {
        var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '/' + resourceType;

        var id = String((0, _schema.getIdFromModel)(model));
        methods.setState((methods.getState() || (0, _immutable.Map)()).withMutations(function (map) {
          map.update('models', function () {
            var models = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.Map)();
            return models.mergeDeep((0, _immutable.fromJS)(_defineProperty({}, id, model)));
          });
          map.updateIn(methods.requestsPath([path, 'data']), function () {
            var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.List)();
            return data.push(id);
          });
        }));
      },

      clearCollection: function clearCollection() {
        var path = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '/' + resourceType;
        return methods.setRequests((0, _immutable.Map)(), path);
      },

      removeFromCollections: function removeFromCollections(map, id) {
        return [].concat(_toConsumableArray((map.getIn(methods.requestsPath()) || (0, _immutable.Map)()).entries())).reduce(function (undo, _ref) {
          var _ref2 = _slicedToArray(_ref, 2),
              path = _ref2[0],
              request = _ref2[1];

          var collection = request.get('data');
          if (_immutable.List.isList(collection)) {
            var modelIdx = collection.findIndex(function (modelId) {
              return modelId === id;
            });
            if (~modelIdx) {
              var nextCollection = collection.delete(modelIdx);
              map.setIn(methods.requestsPath([path, 'data']), nextCollection);
              undo.push(function () {
                return methods.setRequestsData(path, nextCollection.insert(modelIdx, id));
              });
            }
          }
          return undo;
        }, []);
      },

      clearAll: function clearAll() {
        methods.setRequests((0, _immutable.Map)());
        methods.setModels((0, _immutable.Map)());
      },

      subscribePath: rootPath
    };

    return methods;
  }
});