(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'immutable'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('immutable'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.immutable);
    global.schema = mod.exports;
  }
})(this, function (exports, _immutable) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.isSchema = undefined;
  exports.getIdField = getIdField;
  exports.getIdFromModel = getIdFromModel;
  exports.getSchema = getSchema;

  var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
    return typeof obj;
  } : function (obj) {
    return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
  };

  /**
   * example schemas
   *
   * - bare minimum
   *
   * {
   *   "$schema": "http://json-schema.org/draft-04/schema#",
   *   "title": "sources",
   *   "properties": {
   *     "_id": { "type": "string" }
   *   }
   * };
   *
   * title is assumed to be the resource route. the above will get converted into
   * something like the following:
   *
   * sources() // GET '/sources'
   * sources.$create() // POST '/sources'
   * sources({ _id: 'foo' }).$update() // PUT '/sources/foo'
   * sources({ _id: 'foo' }).$delete() // DELETE '/sources/foo'
   *
   */
  function getIdField(schema) {
    return Object.keys(schema.properties).find(function (field) {
      return (/^_?id$/.test(field)
      );
    }) || 'id';
  }

  function getIdFromModel(model) {
    return _immutable.Map.isMap(model) ? model.get('_id') || model.get('id') || model.get('ID') : model._id || model.id || model.ID;
  }

  var isSchema = exports.isSchema = function isSchema(props) {
    return function (key) {
      var maybeSchema = props[key];
      return maybeSchema && (typeof maybeSchema === 'undefined' ? 'undefined' : _typeof(maybeSchema)) === 'object' && (!!maybeSchema.$schema || !!maybeSchema.schema || !!maybeSchema.routes) || typeof maybeSchema === 'string' && maybeSchema.indexOf('/') === 0;
    };
  };

  function getSchema(maybeSchema) {
    return maybeSchema && (typeof maybeSchema === 'undefined' ? 'undefined' : _typeof(maybeSchema)) === 'object' ? maybeSchema.$schema || maybeSchema.schema ? maybeSchema.schema || maybeSchema : maybeSchema.routes ? { title: maybeSchema.routes.default.slice(1) } : null : typeof maybeSchema === 'string' && maybeSchema.indexOf('/') === 0 ? { title: maybeSchema.slice(1) } : null;
  }
});